<?php
/**
 * @package RSDirectory!
 * @copyright (C) 2013-2021 www.rsjoomla.com
 * @license GPL, http://www.gnu.org/licenses/gpl-3.0.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

// Load the RSDirectory! Plugin helper class file if it exists or stop the execution of the script if it doesn't exist.
if ( file_exists(JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/plugin.php') )
{
	require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/plugin.php';
}
else
{
	return;
}

/**
 * RSDirectory! PayFast Payment Plugin.
 */
class plgSystemRSDirectoryPayFast extends RSDirectoryPlugin
{
	const PAYMENT_METHOD = 'payfast';
	const PLUGIN = 'rsdirectorypayfast';
	const EXTENSION = 'plg_system_rsdirectorypayfast';
	const OPTION = 'PLG_SYSTEM_RSDIRECTORYPAYFAST_NAME';

	protected $autoloadLanguage = true;

	/**
	 * Class constructor.
	 *
	 * @access public
	 *
	 * @param object &$subject
	 * @param array $config
	 */
	public function __construct(&$subject, $config)
	{
		parent::__construct($subject, $config);

		// Get mainframe.
		$app = JFactory::getApplication();

		if ( $app->isClient('administrator') )
		{
			$lang = JFactory::getLanguage();
			$lang->load('com_rsdirectory', JPATH_ADMINISTRATOR);

			// Show a warning in the admin section if the PayFast Merchant ID is empty.
			if ( !trim( $this->params->get('merchantid', '') ) )
			{
				$app->enqueueMessage( JText::_('PLG_SYSTEM_RSDIRECTORYPAYFAST_EMPTY_MERCHANT_ID_ERROR'), 'warning' );
			}

			// Show a warning in the admin section if the PayFast Merchant Key is empty.
			if ( !trim( $this->params->get('merchantkey', '') ) )
			{
				$app->enqueueMessage( JText::_('PLG_SYSTEM_RSDIRECTORYPAYFAST_EMPTY_MERCHANT_KEY_ERROR'), 'warning' );
			}
		}
	}

	/**
	 * Can the plugin run?
	 *
	 * @access public
	 *
	 * @return bool
	 */
	public function canRun()
	{
		return parent::canRun() && trim( $this->params->get('merchantid') ) && JPluginHelper::isEnabled('system', self::PLUGIN);
	}

	/**
	 * On after render.
	 *
	 * @access public
	 */
	public function onAfterRender()
	{
		if ( !$this->canRun() )
			return;

		// Get the JInput object.
		$jinput = JFactory::getApplication()->input;

		$is_success = $jinput->getInt('rsdirectorypayfastsuccess');
		$is_cancelled = $jinput->getInt('rsdirectorypayfastcancelled');
		$is_process = $jinput->getInt('rsdirectorypayfastprocess');

		if ($is_success || $is_cancelled || $is_process)
		{
			$this->processForm();
		}
	}

	/**
	 * Add the current payment option to the payments list.
	 *
	 * @access public
	 *
	 * @return string
	 */
	public function onRsdirectoryaddPaymentOptions()
	{
		if ( !$this->canRun() )
			return;

		$tax_text = '';

		if ( $tax_value = $this->params->get('tax_value') )
		{
			if ( $this->params->get('tax_type') )
			{
				$tax_text = "$tax_value%";
			}
			else
			{
				$tax_text = RSDirectoryHelper::formatPrice($tax_value);
			}
		}


		return (object)array(
			'value' => self::PAYMENT_METHOD,
			'text' => JText::_(self::OPTION),
			'tax_text' => $tax_text,
		);
	}

	/**
	 * Show payment form.
	 *
	 * @access public
	 *
	 * @param object $vars
	 */
	public function onRsdirectoryshowForm($vars)
	{
		// Do a few checks and exit the function if the conditions are not met.
		if ( !$this->canRun() || !isset($vars->method) || $vars->method != self::PAYMENT_METHOD )
			return;

		$lang = JFactory::getLanguage();
		$lang->load('com_rsdirectory', JPATH_SITE);


		self::outputTransactionDetails($vars);

		$new_vars = array(
			'merchant_id' 	=> $this->params->get('merchantid'),
			'merchant_key'  => $this->params->get('merchantkey'),
			'return_url' 	=> JRoute::_( JURI::root() . 'index.php?rsdirectorypayfastsuccess=1' ),
			'cancel_url' 	=> JRoute::_( JURI::root() . 'index.php?rsdirectorypayfastcancelled=1' ),
			'notify_url' 	=> JRoute::_( JURI::root() . 'index.php?rsdirectorypayfastprocess=1' ),


			'm_payment_id'  => $vars->hash,
			'amount' 		=> self::formatPrice($vars->total),
			'item_name' 	=> $this->escape($vars->item_name),
		);

		$signature	= '';
		$passphrase		= $this->params->get('pass','');

		foreach ($new_vars as $key => $val) {
			$signature .= $key.'='.urlencode(stripslashes($val)).'&';
		}
		$signature = substr($signature, 0, -1);

		if ($passphrase) {
			$signature .= '&passphrase='.$passphrase;
		}
		$new_vars['signature'] = md5($signature);

		?>

		<p style="margin: 10px; font-weight: bold;"><?php echo JText::_('PLG_SYSTEM_RSDIRECTORYPAYFAST_REDIRECTING'); ?></p>

		<form id="payfastForm" method="post" action="<?php echo ($this->params->get('mode') ? 'https://www.payfast.co.za/eng/process' : 'https://sandbox.payfast.co.za/eng/process');?>">
			<?php foreach ($new_vars as $name => $value) { ?>
				<input type="hidden" name="<?php echo $name; ?>" value="<?php echo $value; ?>" />
			<?php }	?>
		</form>

		<script type="text/javascript">
			function payfastFormSubmit()
			{
				document.getElementById('payfastForm').submit()
			}

			try
			{
				window.addEventListener ? window.addEventListener('load', payfastFormSubmit, false) : window.attachEvent('onload', payfastFormSubmit);
			}
			catch (err)
			{
				payfastFormSubmit();
			}
		</script>

		<?php
	}

	/**
	 * Process the form.
	 *
	 * @access private
	 */
	private function processForm()
	{
		if ( !$this->canRun() )
		{
			return;
		}


		$filter = JFilterInput::getInstance( array(), array(), 1, 1, 0 );
		$jinput  = new JInput( null, array('filter' => $filter) );

		$is_success = $jinput->getInt('rsdirectorypayfastsuccess');
		$is_cancelled = $jinput->getInt('rsdirectorypayfastcancelled');
		$is_process = $jinput->getInt('rsdirectorypayfastprocess');

		if ($is_success)
		{
			$redirect_url = $this->params->get('redirect_url');

			// The redirect_url parameter was not set.
			if (!$redirect_url)
			{
				$redirect_url = JRoute::_('index.php?option=com_rsdirectory&view=myaccount&tab=transactions', false);
			}

			JFactory::getApplication()->redirect($redirect_url);
		}

		if ($is_cancelled)
		{
			$cancel_url = $this->params->get('cancel_url');

			// The redirect_url parameter was not set.
			if (!$cancel_url)
			{
				$cancel_url = JRoute::_('index.php?option=com_rsdirectory&view=myaccount&tab=transactions', false);
			}

			JFactory::getApplication()->redirect($cancel_url);
		}

		// if no process defined than there is no need to go on
		if (!$is_process)
		{
			return;
		}

		// From here the process for the notify is managed
		$m_payment_id = $jinput->get('m_payment_id', '', 'none');

		if ($m_payment_id)
		{
			$hash = $m_payment_id;
		}

		// Exit the function if the hash is empty.
		if ( empty($hash) )
			return;


		// Get the user transaction.
		$user_transaction = RSDirectoryHelper::getUserTransaction($hash);

		// Exit the function if no transaction was found or if the transaction was already finalized.
		if (!$user_transaction || $user_transaction->status == 'finalized') {
			return;
		}

		$email_custom_fields = array(
			'price-credits' 		=> $user_transaction->price,
			'tax' 					=> $user_transaction->tax,
			'tax-type' 				=> $user_transaction->tax_type,
			'tax-value' 			=> $user_transaction->tax_value,
			'total' 				=> $user_transaction->total,
			'credit-title' 			=> $user_transaction->credit_title,
			'currency' 				=> $user_transaction->currency,
			'credits' 				=> $user_transaction->credits,
			'payment-method' 		=> $user_transaction->gateway,
			'date-purchased' 		=> $user_transaction->date_created,
		);

		$send_failed_email = true;

		$post = JFactory::getApplication()->input->post->getArray();
		$postData = array();

		foreach ($post as $key => $value) {
			if ($key != 'signature') {
				$postData[] = $key."=".urlencode(stripslashes($value));
			}
		}
		
		$postData = implode('&', $postData);
		$postfields = $postData;
		$passphrase	= $this->params->get('pass','');

		if (!empty($passphrase)) {
			$postfields .= '&passphrase='.urlencode($passphrase);
		}

		$signature = md5($postfields);

		// include the email helper
		require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/email.php';

		// Initialize the log array.
		$log = array();

		$log[] = "Receiving a new transaction from PayFast.";

		if ($post['signature'] == $signature) {
			if ( $this->params->get('mode', 0) == 0 )
			{
				$log[] = 'Test mode is on.';
			}

			$host = $this->params->get('mode') ? 'www.payfast.co.za' : 'sandbox.payfast.co.za';

			$http = JHttpFactory::getHttp();

			// hold the connection response
			$response = '';

			// hold the connection errors
			$errstr = '';
			try
			{	
				$request = $http->post('https://'.$host.'/eng/query/validate', $postData, array(), 5);

				$code = $request->code;

				if ($code != 200)
				{
					throw new Exception(sprintf('Connection Error: %d', $request->code));
				}

				// set the response
				$response = (string) $request->body;

				if (!strlen($response))
				{
					throw new Exception('Response does not contain a valid body');
				}
			}
			catch (Exception $e)
			{
				$errstr = $e->getMessage();
			}

			if (!strlen($response))
			{
				$log[] = "Could not connect to $host in order to verify this transaction. Error reported is: $errstr";
			}
			else
			{
				$lines = explode("\n", $response);
				$result = trim($lines[0]);
				
				if (strcmp( $result, 'VALID' ) == 0 )
				{
					// Format the credits package price.
					$total = self::formatPrice($user_transaction->total);

					$log[] = 'PayFast reported a valid transaction.';
					$log[] = 'Payment status is ' . ( !empty($post['payment_status']) ? $post['payment_status'] : 'empty' ) . '.';

					if ($total == $post['amount_gross'])
					{
						// remove the posts that are empty
						$new_post = array();
						foreach($post as $key => $val)
						{
							$val = trim($val);
							if (!empty($val))
							{
								$new_post[$key] = $key.': '.$val;
							}
						}

						$new_user_transaction = (object)array(
							'id' => $user_transaction->id,
							'gateway_order_number' => $post['pf_payment_id'],
							'gateway_params' => !empty($new_post) ? implode("\n", $new_post) : '',
							'date_finalized' => JFactory::getDate()->toSql(),
							'finalized' => 1,
							'status' => 'finalized',
						);

						$db = JFactory::getDbo();

						$db->updateObject('#__rsdirectory_users_transactions', $new_user_transaction, 'id');

						RSDirectoryCredits::addUserCredits($user_transaction->user_id, $user_transaction->credits);

						// set the email custom fields
						$email_custom_fields['gateway-order-number'] = $new_user_transaction->gateway_order_number;
						$email_custom_fields['gateway-order-type'] 	 = '';
						$email_custom_fields['date-finalized'] 		 = $new_user_transaction->date_finalized;
						$email_custom_fields['status'] 				 = JText::_('COM_RSDIRECTORY_TRANSACTION_STATUS_'.strtoupper($new_user_transaction->status));


						// Get entry.
						if ($user_transaction->entry_id && ($entry = RSDirectoryHelper::getEntry($user_transaction->entry_id))) {
							// Get the number of unpaid entry credits.
							$cost = RSDirectoryCredits::getUnpaidEntryCreditsSum($entry->id);

							// Get the total number of credits available to the user
							$user_credits = RSDirectoryCredits::getUserCredits($user_transaction->user_id);

							// Get plugin entry publishing param.
							$publish_entry = (int) $this->params->get('publish_entry', 0);

							// Verify if this entry can be published automatically.
							$automatically_publish_entry = $publish_entry == 1 || ($publish_entry == 2 && RSDirectoryHelper::checkUserPermission('auto_publish_entries', $user_transaction->user_id));

							// Check if enough credits are available.
							$has_enough_credits = $user_credits == 'unlimited' || $cost <= $user_credits;

							if ($automatically_publish_entry && $has_enough_credits) {
								// Holds emails that will be sent later on.
								$emails = array('credit_purchase_processed_finalized_with_entry');

								// Setup new entry data that should be overridden.
								$entry_data = (object)array(
									'id' => $entry->id,
									'paid' => 1,
									'published' => 1,
									'published_time' => JFactory::getDate()->toSql()
								);

								// Calculate expiry time.
								if ($entry->period) {
									$expiry_time = JFactory::getDate($entry_data->published_time);
									$expiry_time->modify("+{$entry->period} days");

									$entry_data->expiry_time = $expiry_time->toSql();
								} else {
									$entry_data->expiry_time = $db->getNullDate();
								}

								// Flag to send emails if the entry wasn't published and now it is.
								if (!$entry->published) {
									$emails[] = 'publish_entry';
								}

								// Subtract number of credits if the user doesn't have unlimited credits
								if ($user_credits != 'unlimited') {
									// Charge user.
									$query = $db->getQuery(true)
										->update($db->qn('#__rsdirectory_users'))
										->set($db->qn('credits') . ' = ' . $db->qn('credits') . ' - ' . $db->q($cost))
										->where($db->qn('user_id') . ' = ' . $db->q($user_transaction->user_id));

									$db->setQuery($query);
									$db->execute();
								}

								// Mark entry credits as being paid.
								$query = $db->getQuery(true)
									->update($db->qn('#__rsdirectory_entries_credits'))
									->set($db->qn('paid') . ' = ' . $db->q($entry_data->paid))
									->where($db->qn('entry_id') . ' = ' . $db->q($entry_data->id));

								$db->setQuery($query);
								$db->execute();

								// Save entry data.
								$db->updateObject('#__rsdirectory_entries', $entry_data, 'id');

								$log[] = 'Automatically published entry #'.$entry->id.'.';

								// Send emails.
								if ($emails) {
									// Get the form associated to the category.
									$form = RSDirectoryHelper::getCategoryInheritedForm($entry->category_id);

									// Get the published form fields assigned to the form.
									$form_fields = RSDirectoryHelper::getFormFields($form->id, 1, 1);

									// Send emails
									foreach ($emails as $email) {
										RSDirectoryEmail::getInstance($email, $form_fields, $entry, $form, ($email == 'credit_purchase_processed_finalized_with_entry' ? $email_custom_fields : null))->send();
										$log[] = "Sent $email.";
									}
								}
							} else {
								// Get the form associated to the category.
								$form = RSDirectoryHelper::getCategoryInheritedForm($entry->category_id);

								// Get the published form fields assigned to the form.
								$form_fields = RSDirectoryHelper::getFormFields($form->id, 1, 1);

								RSDirectoryEmail::getInstance('credit_purchase_processed_finalized_with_entry', $form_fields, $entry, $form, $email_custom_fields)->send();
								$log[] = "Sent credit_purchase_processed_finalized_with_entry.";
							}
							$send_failed_email = false;
						} else {
							$entry = (object) array('skip_entry'=>true, 'user_id' => $user_transaction->user_id);
							RSDirectoryEmail::getInstance('credit_purchase_processed_finalized_no_entry', null, $entry, null, $email_custom_fields)->send();
							$log[] = "Sent credit_purchase_processed_finalized_no_entry.";
							$send_failed_email = false;
						}
					}
					else
					{
						$log[] = "Expected an amount of $total $user_transaction->currency. PayFast reports this payment is ".$post['amount_gross']." ZAR. Stopping";

					}

				} else {
					$log[] = "Could not verify transaction authenticity. PayFast said it's invalid.";
					$log[] = "String sent to PayFast is $postData";
				}
			}
		}
		else
		{
			$log[] = "Could not verify transaction authenticity. Security signature mismatch.";
		}

		// send the refusal email if is set
		if ($send_failed_email) {
			if ($user_transaction->entry_id && ($entry = RSDirectoryHelper::getEntry($user_transaction->entry_id))) {
				// Get the form associated to the category.
				$form = RSDirectoryHelper::getCategoryInheritedForm($entry->category_id);

				// Get the published form fields assigned to the form.
				$form_fields = RSDirectoryHelper::getFormFields($form->id, 1, 1);

				RSDirectoryEmail::getInstance('credit_purchase_processed_refused_with_entry', $form_fields, $entry, $form, $email_custom_fields)->send();
			} else {
				$entry = (object) array('skip_entry'=>true, 'user_id' => $user_transaction->user_id);
				RSDirectoryEmail::getInstance('credit_purchase_processed_refused_no_entry', null, $entry, null, $email_custom_fields)->send();
			}
		}

		RSDirectoryHelper::saveTransactionLog($user_transaction->id, $log);

	}

	/**
	 * Get taxa data.
	 *
	 * @access public
	 *
	 * @param object $vars
	 *
	 * @return array
	 */
	public function onRsdirectorygetTaxData($vars)
	{
		if ( !$this->canRun() || !isset($vars->method) || $vars->method != self::PAYMENT_METHOD )
			return;

		// Calculate the tax.
		$tax_type = $this->params->get('tax_type');
		$tax_value = $this->params->get('tax_value');
		$tax = $tax_value && $tax_type ? $vars->price * ($tax_value/100) : $tax_value;

		$data = array(
			'tax_type' => $tax_type ? 'percent' : 'fixed',
			'tax_value' => $tax_value,
			'tax' => $tax,
		);

		return $data;
	}
}